(function(window) {
	"use strict";
	var scraper = window.scraper;
	var self = scraper.worker = {};

	var BASE_URL = "https://banking.barclaycard.de/bir/feature";
	var START_URL = BASE_URL + "/loginprocess";
	var UMSAETZE_URL = BASE_URL + "/creditCardDetail?acctIndex=0";
	var logoutUrl;

	self.bpdVersion = 2;
	self.updVersion = 1;

	function checkServerfehler() {
		var m = scraper.getText().match(/Es ist ein Fehler aufgetreten. Bitte versuchen Sie es zu einem späteren Zeitpunkt erneut./);
		if (m) scraper.hirmsThrow("Serverfehler: " + m[0]);
	}

	self.login = function() {
		scraper.httpGet("start", START_URL);
		var m0 = scraper.findForms("loginForm")[0].map;
		var map = {
			CSRF_TOKEN: m0.CSRF_TOKEN,
			"loginForm:loginName": scraper.data.kennung,
			"loginForm:password": scraper.data.pin,
			_t: m0._t,
			loginForm_SUBMIT: "1",
			"javax.faces.ViewState": m0["javax.faces.ViewState"],
			forward: "true",
			"loginForm:_idcl": "loginForm:loginLink"
		};
		scraper.httpPost("sendForm-loginForm", m0.action, map);
		logoutUrl = scraper.findSimple("logout", /\"(\/bir\/feature\/logout)\"/);
	};

	self.logout = function() {
		scraper.httpGet("logout", logoutUrl);
	};

	self.kontoliste = function() {
		var t = scraper.findSimple("kontoliste", /<h2>Ihr Barclaycard Kreditkarten-Konto((?:\n.*)+?)Klicken Sie bitte auf eine Kontonummer, um Details zu einem Konto anzuzeigen./);
		t = t.replace(/\s+/g, " ").replace(/>\s*</g, ">\n<").trim();
		t = t.split("\n");
		var i = 0;
		while (!t[i].match(/<a href=\"(creditCardDetail\?acctIndex=\d+)\">/)) ++i;
		while (!t[i].match(/<td>/)) ++i;
		var bezeichnung = t[i].match(/<td>(.*)<\/td>/)[1];
		++i;
		var saldo = t[i].match(/<td>(.*)<\/td>/)[1];
		var waehrung = saldo.substring(0, 3);
		while (t[i] !== "<span>Ablaufdatum</span>") ++i;
		while (!t[i].match(/<td /)) ++i;
		var kontoNr = t[i].match(/<td [^>]+>(.*)<\/td>/)[1];
		return [{kontoNr: kontoNr, waehrung: waehrung, bezeichnung: bezeichnung}];
	};

	self.umsaetze = function() {
		(function getCsv() {
			scraper.httpGet("umsaetze-start", UMSAETZE_URL);
			checkServerfehler();
			var m0 = scraper.findForms("detailForm")[0].map;
			m0 = {
				CSRF_TOKEN: m0.CSRF_TOKEN,
				"detailForm:accountList": "0",
				"detailForm:FILTER_TYPE": "3",
				"detailForm_SUBMIT": "1",
				"javax.faces.ViewState": m0["javax.faces.ViewState"],
				processIds : "*",
				cursorNormal: "true",
				pFragments: "detailForm:creditUnbillPrimaryTable",
				forward: "false",
			};
			scraper.httpPost("sendForm-umsaetzeFragment", m0.action, m0);
			checkServerfehler();

			var detailForm = scraper.findSimple("csv-link", /[\s\S]+'(detailForm:j_id\d+)'[\s\S]+Export to XLS format/);
			var m1 = scraper.findForms("detailForm")[0].map;
			m1 = {
				CSRF_TOKEN: m1.CSRF_TOKEN,
				"detailForm:accountList": "0",
				"detailForm:FILTER_TYPE": "3",
				"detailForm_SUBMIT": "1",
				"javax.faces.ViewState": m1["javax.faces.ViewState"],
				cursorWait : "false",
				"detailForm:_idcl": detailForm,
			};
			scraper.httpPost("sendForm-umsaetzeCsv", m1.action, m1);
			checkServerfehler();
		})();

		function toNumber(s) {
			var sign = +1;
			if (s.charAt(s.length-1) === '-') {
				sign = -1;
				s = s.substring(0, s.length-1);
			} else if (s.charAt(s.length-1) === '+') {
				s = s.substring(0, s.length-1);
			}
			return sign * s.replace(/\./g, "").replace(",", ".");
		}

		function toDatum(s) {
			var m = s.match(/^(\d+)\.(\d+)\.(\d+)$/);
			while (m[3].length < 4) m[3] = '0' + m[3];
			while (m[2].length < 2) m[2] = '0' + m[2];
			while (m[1].length < 2) m[1] = '0' + m[1];
			return m[3] + m[2] + m[1];
		}

		var endSaldo, umsaetze;
		var t = scraper.getText().split(/\s*\n\s*/);
		for (var i=0; i<t.length; ++i) {
			var s = t[i].trim();
			console.log("line", s);
			if (!s) continue;
			if (s.charAt(0) != "\"") continue;
			if (s.charAt(s.length-1) != "\"") continue;
			var fields = s.substring(1, s.length-1).split(/\",\"/);
			if (umsaetze) {
				umsaetze.push({
					gebucht: true,
					abgerechnet: true,
					valuta: toDatum(fields[0]),
					datum: toDatum(fields[1]),
					zweck: fields[2],
					betrag: toNumber(fields[5]),
					waehrung: "EUR",
				});
			} else if (fields[0] === "Buchung/ Valuta") {
				umsaetze = [];
			} else if (fields[0] === "Aktueller Saldo") {
				var d = new Date();
				endSaldo = {
					betrag: toNumber(fields[1].substring(4)),
					waehrung: fields[1].substring(0, 3),
					datum: toDatum(d.getDate() + "." + (d.getMonth() + 1) + "." + d.getFullYear())
				};
			}
		}
		scraper.assertTrue("has-umsaetze", umsaetze);
		scraper.assertTrue("has-endSaldo", endSaldo);
		return {umsaetze: umsaetze, endSaldo: endSaldo};
	};
})(window);
