(function(window) {
	"use strict";
	var scraper = window.scraper;
	var self = scraper.worker = {};

	var BASE_URL = "https://www.dkb.de";
	var START_URL = BASE_URL + "/banking";
	var KK_URL = START_URL + "/finanzstatus/kreditkartenumsaetze";
	var logoutUrl;

	scraper.settings.overlapDays = scraper.settings.dkbVisaOverlapDays;

	self.bpdVersion = 16;
	self.updVersion = 1;

	self.login = function() {
		scraper.httpGet("start", START_URL);
		scraper.sendForm("login", null, {j_username: scraper.data.kennung, j_password: scraper.data.pin});
		logoutUrl = scraper.resolveLink("LogoutLink", /href="(\/DkbTransactionBanking\/[^\"]+\$event=logout)"/);

		function parseChooseTanAliasForm(text) {
			var result = {};
			var regex = /<label for="slDeviceNames:(\d+)"[^<>]*\>\s*(.+?)\s*<\/label>/g;
			while (1) {
				var m = regex.exec(text);
				if (!m) break;
				result[m[2].toLowerCase()] = m[1];
			}
			return result;
		}

		function getMediumIndex(medium, parsedForm) {
			var result = parsedForm[medium.toLowerCase()];
			if (result) return result;
			scraper.hirmsAdd("3999 Keine exakte Uebereinstimmung fuer den TAN-Medium-Namen: " + medium);
			var parsedFormKeys = Object.keys(parsedForm);
			var bestDistance = 1000;
			for (var i=0; i<parsedFormKeys.length; ++i) {
				var d = wordDistance(medium, parsedFormKeys[i]);
				if (d  >= bestDistance) continue;
				bestDistance = d;
				result = parsedFormKeys[i];
			}
			return result;
		}

		function wordDistance(a, b) {
			result = 0;
			if (a===b) return 0;
			if (a.length===0 || b.length===0) return 1000;
			result += a[0]!==b[0] ? 1 : 0;
			result += a[a.length-1]!==b[b.length-1] ? 1 : 0;
			result += Math.abs(a.length - b.length) / 2;
			return result;
		}

		var name;
		var predicate = function(f) {return f.name!=="backForm" && f.action.indexOf(name) >= 0;};
		while (true) {
			{
				name = "InfoOpenLoginRequest";
				var form = scraper.findForms(name, predicate)[0];
				println(name + "=" + (form ? 1 : 0));
				if (form) {
					form();
					continue;
				}
			}
			{
				name = "LoginWithBoundDevice";
				var form = scraper.findForms(name, predicate)[0];
				println(name + "=" + (form ? 1 : 0));
				if (form) {
					var hint = "Bitte bestätigen Sie den Auftrag in Ihrer DKB-Banking-App und klicken Sie auf \"OK\".\n\n"
							+ "Oder klicken Sie auf \"OK\" ohne die Bestätigung, um zu der TAN-Eingabe zu kommen.";
					scraper.query(hint, scraper.queryType.TAN_NONE);
					form();
					continue;
				}
			}
			{
				name = "LoginWithTanSubmit";
				var form = scraper.findForms(name, predicate)[0];
				println(name + "=" + (form ? 1 : 0));
				if (form) {
					var query = getQuery();
					var tan = scraper.query(query.hint, query.type);
					if (tan) {
						scraper.sendForm(name, predicate, {tan: tan, $event: "next"});
						continue;
					}
				}
			}
			{
				name = "chooseTanAlias";
				var form = scraper.findForms(name, predicate)[0];
				println(name + "=" + (form ? 1 : 0));
				if (form) {
					var query = {hint: "TAN2go Endgerät auswählen", type: 'TAN_CHOOSE_MEDIUM'};
					var medium = scraper.query(query.hint, query.type).trim();
					if (medium) {
						var slDeviceNames = getMediumIndex(medium, parseChooseTanAliasForm(form.text)) - 1;
						scraper.sendForm(name, predicate, {slDeviceNames: slDeviceNames, $event: "next"});
						continue;
					}
				}
			}
			{
				name = "SynchronizeChipTANGeneratorProcess";
				var form = scraper.findForms(name, predicate)[0];
				if (form) hirmsThrow("Bitte führen Sie die Synchronisierung-Synchronisierung einmalig auf der Webseite der Bank aus.");
			}
			println("No known form, hoping for the best.");
			return;
		}
	};

	function getQuery() {
		var text = scraper.getText();
		var m = text.match(/>Startcode (\d+)</);
		if (m) {
			return {hint: m[1], type: scraper.queryType.TAN_STARTCODE};
		} else if (text.match(/TAN aus Ihrer DKB-TAN2go-App/)) {
			var hint = "Bitte bestätigen Sie Ihren Auftrag mit der TAN aus Ihrer DKB-TAN2go-App.";
			return {hint: hint, type: scraper.queryType.TAN_PUSHTAN};
		} else {
			var hint = "moneyplex konnte die TAN-Aufforderung der Bank nicht erkennen...";
			return {hint: hint, type: scraper.queryType.TAN_NONE};
		}
	}

	self.logout = function() {
		scraper.httpGet("start", logoutUrl);
		scraper.findSimple("logout-ok", /(erfolgreich abgemeldet)/);
	};

	self.kontoliste = function() {
		return null;
	};

	self.umsaetze = function() {
		var kontoNr = scraper.data.kontoNr;

		scraper.httpGet("start", KK_URL + "?$event=init");
		function getOptions() {
			return scraper.findSimple("select", /<select\b[^<>]*\bname="slAllAccounts" [^<>]+>([\s\S]*?)<\/select>/);
		}

		var slAllAccounts = (function() {
			var shouldVisitWebPage = !!~scraper.getText().indexOf("Warum sehe ich meinen Finanzstatus nicht?");
			if (shouldVisitWebPage) scraper.query("Melden Sie sich bitte an der Webseite der Bank an.", "INFO");
			var options;
			try {
				options = getOptions();
			} catch (e) {
				scraper.httpGet("trying-FinancialStatus", "/DkbTransactionBanking/content/banking/financialstatus/FinancialComposite/FinancialStatus.xhtml?$event=init");
				options = getOptions();
			}
			//console.log("options", options);
			var p = 'value="([^\"]+)"[^<>]*>\\s*' + kontoNr.replace(/^(.{4}).*(.{4})$/, "$1\\**$2");
			var m = options.match(new RegExp(p));
			scraper.assertTrue("slAllAccounts", m);
			return m[1];
		})();
		var postingDate = (function() {
			var a = new Date();
			a.setTime(a.getTime() - 365*24*60*60*1000);
			var b = scraper.hbciDateToDate(scraper.data.von) || a;
			return scraper.formatDate4(a > b ? a : b);
		})();
		var toPostingDate = (function() {
			var a = scraper.hbciDateToDate(scraper.data.bis) || new Date();
			return scraper.formatDate4(a);
		})();
		var csv = (function() {
			var csv = [];
			for (var i=0; i<2; ++i) {
				scraper.httpPost("read", KK_URL, {
					hiddenSubmit: "",
					slAllAccounts: slAllAccounts,
					slTransactionStatus: i,
					slSearchPeriod: 0,
					filterType: "DATE_RANGE",
					postingDate: postingDate,
					toPostingDate: toPostingDate,
					$event: "search",
				});
				scraper.httpGet("csv", KK_URL + "?$event=csvExport");
				csv.push(scraper.getText());
			}
			return csv;
		})();
		//console.log(csv[0]);
		//console.log(csv[1]);

		function match1(text, regex) {
			var m = text.match(regex);
			scraper.assertTrue("match1", m);
			return m[1];
		}

		function reformatDate(d) {
			return d.replace(/^(\d+)\.(\d+)\.(\d+)$/, "$3$2$1");
		}

		function getUmsatze(csv, gebucht) {
			var list = [];
			var i = csv.indexOf("Umsatz abgerechnet und nicht im Saldo enthalten");
			scraper.assertTrue("csvHeader", i > -1);
			csv = csv.substring(i);
			i = csv.indexOf("\n");
			if (i == -1) return list;
			csv = csv.substring(i+1);
			if (!csv) return [];
			var lines = csv.split("\n");
			for (var j=0; j<lines.length; ++j) {
				var line = match1(lines[j], /^"(.*)";$/).split('";"');
				scraper.assertTrue("lineLength", line.length >= 5);
				var umsatz = {
					gebucht: gebucht,
					abgerechnet: line[0] === "Ja",
					valuta: reformatDate(line[1]),
					datum: reformatDate(line[2]),
					zweck: (line[3].replace(/\s+/g, " ") + "\n" + line[5]).trim(),
					betrag: +(line[4].replace(/\./g, "").replace(",", ".")),
					waehrung: 'EUR',
				};
				list.push(umsatz);
			}
			return list;
		}

		var result = {
			umsaetze: getUmsatze(csv[0], true).concat(getUmsatze(csv[1], false)),
		};

		result.endSaldo = (function() {
			var m = csv[1].match(/"Saldo:";"(-?[.\d]+) ([A-Z]{3})";/);
			scraper.assertTrue("endSaldo-match", m);
			var betrag = +m[1];
			var waehrung = m[2];

			m = csv[1].match(/"Datum:";"([\d]+\.[\d]+\.[\d]+)";/);
			m = m || csv[1].match(/"Bis:";"([\d]+\.[\d]+\.[\d]+)";/);
			scraper.assertTrue("endSaldo-datum", m);
			var datum = reformatDate(m[1]);
			return {betrag: betrag, waehrung: waehrung, datum: datum};
		})();

		result.anfangsSaldo = (function() {
			var betrag = result.endSaldo.betrag;
			for (var i=0; i<result.umsaetze.length; ++i) {
				var u = result.umsaetze[i];
				betrag -= u.betrag;
			}
			var waehrung = result.endSaldo.waehrung;
			var datum = match1(csv[1], /"Von:";"([\d]+\.[\d]+\.[\d]+)";/);
			datum = reformatDate(datum);
			return {betrag: betrag, waehrung: waehrung, datum: datum};
		})();

		return result;
	};
})(window);
