(function(window) {
	"use strict";
	var scraper = window.scraper;
	var self = scraper.worker = {};

	var START_URL = "https://kreditkarten-banking.lbb.de/lbb";
	var logoutUrl;
	var kontoNr;
	var endSaldo;

	self.bpdVersion = 2;
	self.updVersion = 1;

	self.login = function() {
		scraper.httpGet("start", START_URL);
		scraper.sendForm("preLogonForm", null, {user: scraper.data.kennung, password: scraper.data.pin});
		scraper.sendForm("service");

		logoutUrl = scraper.resolveLink("LogoutLink", /<a href="([^\"]+;bt_LOGOUT=do)"/);

		var s = scraper.findSimple("Kreditkartenkonto", /Kreditkartenkonto:<\/td>\s+<td class="tabdata">\s+<a [^<]+>([^<]+)<\/a>/);
		//println("Kreditkartenkonto", s);
		kontoNr = s.replace(/\s/g, "").replace(/x/gi, "*");
		//println("Kreditkartenkonto-2", kontoNr);

		endSaldo = {};
		s = scraper.findSimple("Kontostand", /Kontostand:<\/td>\s+<td class="tabtext">([^<]+)/);
		var m = s.match(/\s*([A-Z]{3})\s*&nbsp;\s*([-+]?[.,\d]+)\s*/);
		scraper.assertTrue("match-Kontostand", m);
		endSaldo.betrag = +(m[2].replace("+", "").replace(".", "").replace(",", "."));
		endSaldo.waehrung = m[1];
		s = scraper.findSimple("Kontostand-vom", /Kontostand vom:<\/td>\s+<td class="tabdata">\s*(\d{2}\.\d{2}\.\d{4})\d*<\/td>/);
		m = s.match(/^([^.]+)\.([^.]+)\.([^.]+)$/);
		scraper.assertTrue("match-vom", m);
		endSaldo.datum = m[3] + m[2] + m[1];
	};

	self.logout = function() {
		scraper.httpGet("logout", logoutUrl);
	};

	self.kontoliste = function() {
		return [{kontoNr: kontoNr, waehrung: "EUR", bezeichnung: "Kreditkarte"}];
	};

	self.umsaetze = function() {
		function reformatDate(d) {
			return d.replace(/^(\d+)\.(\d+)\.(\d+)$/, "$3$2$1");
		}

		var umsaetze = [];

		function collect(text, abgerechnet) {
			var foundOld, foundNew;
			var list = text.split(/\r?\n/);
			var last;
			for (var i=0; i<list.length; ++i) {
				var s = list[i].trim();
				if (!s) continue;
				var line = s.split(";");
				var valuta = reformatDate(line[1]);
				var tooOld = scraper.data.von && valuta < scraper.data.von;
				foundOld |= tooOld;
				foundNew |= !tooOld;
				if (tooOld) continue;
				var accepted = scraper.acceptKontoNr(line[0]);
				if (!accepted) println("Ignoring line", "kontoNr=" + kontoNr, "line=" + list[i].trim());
				if (!accepted) continue;
				var umsatz = {
					gebucht: true,
					abgerechnet: abgerechnet,
					valuta: valuta,
					datum: reformatDate(line[2]),
					zweck: line[3],
					betrag: +(line[6].replace(/\./g, "").replace(",", ".")),
					waehrung: "EUR",
				};
				if (!last || umsatz.betrag) {
					umsaetze.push(umsatz);
					last = umsatz;
				} else {
					last.zweck += "@" + umsatz.zweck;
				}
			}
			return foundNew || !foundOld;
		}

		function getNeu() {
			scraper.followLink("umsaetze", /<a href="([^\"]+)" [^>]+>Kartenums&auml;tze<\/a>/);
			scraper.assertTrue("not-match-Verbindung", !scraper.getText().match("Verbindung zum Bankrechner"));
			if (scraper.getText().match("Sie haben seit (Ihrer letzten Kreditkartenabrechnung vom|dem .* keine neuen Kartenums)")) return;
			scraper.followLink("csv", /<a href="([^\"]+bt_TXNCSV=do[^\"]+)"/);
			collect(scraper.getText(), false);
			scraper.restorePreviousState();
		}

		function getAlt() {
			scraper.followLink("Rechnungen", /<a href="([^\"]+bt_STMTLIST=do)" /);

			var forms = scraper.findForms("dispatchForm", function(f) {
				//println("getAlt-findForms", JSON.stringify(Object.keys(f)), f.name, JSON.stringify(f.map));
				return f.name === "dispatchForm" && f.map.bt_STMT;
			});

			for (var i=0; i<forms.length; ++i) {
				forms[i]();
				var forms2 = scraper.findForms("statementForm", null, {}, "bt_STMTSAVE");
				var f = forms2[0];
				scraper.assertTrue("match-form", f);
				f();
				scraper.followLink("csv-" + i, /href="([^\"]+\bbt_STMTCSV=[^\"]+)"/);
				var found = collect(scraper.getText(), true);
				scraper.restorePreviousState();
				if (!found) break;
			}
		}

		getNeu();
		getAlt();

		return {endSaldo: endSaldo, umsaetze: umsaetze};
	};
})(window);
