(function(window) {
	"use strict";
	var scraper = window.scraper;
	var self = scraper.worker = {};

	var BASE_URL = "https://www.miles-and-more.kartenabrechnung.de";
	var logoutUrl;

	self.bpdVersion = 2;
	self.updVersion = 1;

	self.login = function() {
		scraper.httpGet("start", BASE_URL);
		if (!scraper.getText().match(/event=logout/)) {
			scraper.followLink("Redirect", /<a href="([^\"]+FinancialStatus.xhtml)">/);
		}
		scraper.sendForm("login", null, {j_username: scraper.data.kennung, j_password: scraper.data.pin, "$event": "login"});

		if (scraper.getText().match("Passwort ist falsch")) throw scraper.hirmsThrowPassword();
		logoutUrl = scraper.resolveLink("LogoutLink", /href="(\/mam\/(?:Welcome|Home)\/login\.xhtml\?\$event=logout)"/);
	};

	self.kontoliste = function() {
		{
			var text = scraper.getText();
			if (!text.match(/<span class="cardName">/)) {
				scraper.followLink("zwischenseite-kontoliste", /(\/mam\/Home\/content\/FinancialStatus\/Overview.xhtml\?\$event=init)/);
			}
		}
		var kontoNr = scraper.findSimple("kontoNr", /<span class="cardName"><span class="leftPart">[^<>]+<\/span>[\/\* ]+(\d+)<\/span>/);
		while (kontoNr.length < 12) kontoNr = "*" + kontoNr;
		return [{kontoNr: kontoNr, waehrung: "EUR", bezeichnung: "Kreditkarte"}];
	};

	self.umsaetze = function() {
		var csv = (function() {
			{
				var text = scraper.getText();
				if (!text.match(/href="([^\"]+event=showTransactions[^\"]*)"/)) {
					scraper.followLink("zwischenseite-umsaetze", /(\/mam\/Home\/content\/FinancialStatus.xhtml\?\$event=overview)/);
				}
			}
			scraper.followLink("showTransactions", /href="([^\"]+event=showTransactions[^\"]*)"/);
			var result;
			var params = {
				postingDate: (function() {
					var a = new Date();
					a.setTime(a.getTime() - 365*24*60*60*1000);
					var b = scraper.hbciDateToDate(scraper.data.von) || a;
					return scraper.formatDate4(a > b ? a : b);
				})(),
				toPostingDate: (function() {
					var a = scraper.hbciDateToDate(scraper.data.bis) || new Date();
					return scraper.formatDate4(a);
				})(),
				customSearchDateRange: true,
				slUnbookedTransactions: 0, // Reads both booked and unbooked.
				$event: "search",
			};
			scraper.sendForm("transactionSearchForm", null, params);
			if (scraper.getText().match(/den angegebenen Zeitraum sind keine Ums/)) {
				result = "";
			} else {
				scraper.followLink("csvExport", /href="([^\"]+event=csvExport)"/);
				result = scraper.getText();
			}
			scraper.restorePreviousState();
			return result;
		})();

		var result = {
			umsaetze: [],
			endSaldo: {waehrung: "EUR"},
		};

		function reformatDate(d) {
			return d.replace(/^(\d+)\.(\d+)\.(\d+)$/, "$3$2$1");
		}

		function toBetrag(s) {
			return +(s.replace(/\./g, "").replace(",", "."));
		}

		function fixUmsatz(u) {
			u.gebucht = u.valuta !== "vorgemerkt" && u.datum !== "vorgemerkt";
			if (u.valuta === "vorgemerkt") u.valuta = u.datum;
			if (u.datum === "vorgemerkt") u.datum = u.valuta;
			if (u.valuta === "vorgemerkt") {
				u.valuta = u.datum = scraper.reformatDate(new Date(), "", "Ymd");
			}

			if (u.valuta.substring(0, 4) == u.datum.substring(0, 4)) return;
			if (!u.zweck.match(/^Jahresgeb/)) return;
			u.datum = u.valuta;
		}

		function getUmsatze(csv) {
			if (!csv) return;

			if (true) {
				var e = result.endSaldo;
				var m = csv.match(/\n"Saldo:";"([^\"]+)"/);
				scraper.assertTrue("endSaldo-betrag", m);
				e.betrag = toBetrag(m[1]);
				m = csv.match(/\n"Zeitraum:";"[^\" ]+ - ([^\"]+)"/);
				scraper.assertTrue("endSaldo-datum", m);
				e.datum = reformatDate(m[1]);
			}

			csv = csv.split(/\s*\n/);
			var i = 0;
			for (; i<csv.length && !csv[i].match(/Belegdatum/); ++i);
			scraper.assertTrue("csvHeader", i < csv.length);
			for (++i; i<csv.length; ++i) {
				var line = csv[i].replace(/^"(.*)";$/, "$1").split(/";"/);
				scraper.assertTrue("lineLength", line.length >= 7);
				var u = {
					gebucht: true,
					abgerechnet: line[6] === "Ja",
					valuta: reformatDate(line[1]),
					datum: reformatDate(line[0]),
					zweck: line[2].replace(/\s+/g, " "),
					betrag: toBetrag(line[3]),
					waehrung: 'EUR',
				};
				fixUmsatz(u);
				if (!u.gebucht) result.endSaldo.betrag -= u.betrag;
				result.umsaetze.push(u);
			}
		}

		getUmsatze(csv);
		return result;
	};

	self.logout = function() {
		scraper.httpGet("start", logoutUrl);
		scraper.findSimple("logout-ok", /(erfolgreich abgemeldet)/);
	};


})(window);
