(function(window) {
	"use strict";
	var scraper = window.scraper;
	var self = scraper.worker = {};

	var LOGIN0_URL = "https://securebank.santander.de/IRALOG_UNILOGON_ENS/BtoChannelDriver.ssobto?dse_operationName=UniLogon";
	var LOGIN1_URL = "https://securebank.santander.de/IRALOG_UNILOGON_ENS/channel.ssobto?dse_contextRoot=true";

	var OWN_URL = "https://pintan.santanderbank.de";
	var LOGOUT_URL = OWN_URL + "/IRALOG_SEB_ENS/BtoChannelDriver.ssobto?dse_operationName=OP_logoff&logoffParam.type=L&logoffParam.sign=PINTRAVIC";
	var KONTOLISTE_URL = OWN_URL + "/EBANDE_PGSEB_ENS/BtoChannelDriver.ssobto?dse_operationName=globalPositionOwnerOP";

	self.bpdVersion = 2;
	self.updVersion = 1;

	self.login = function() {
		scraper.httpGet("start", LOGIN0_URL);
		scraper.setReferer(LOGIN0_URL);
		scraper.sendForm("loginDataForm", null, {
			"PersonenNummer": null,
			"loginDataForm.personenNummer": scraper.data.kennung,
			"loginDataForm.pin": scraper.data.pin,
		});

		scraper.setReferer(LOGIN1_URL);
		scraper.sendForm("login2", function() {return true}, {
			"loginDataForm.personNumber": scraper.data.kennung,
			"loginDataForm.PIN": scraper.data.pin,
		});
		scraper.setReferer("");
		if (scraper.getText().match(/fehlgeschlagen\b/i)) throw "Login error";

		//!scraper.setReferer("");
		scraper.followLink("finanzstatus", /<a href="([^\"]+)"[^>]*>(?:Zum )?Finanzstatus<\/a>/);
	};

	self.logout = function() {
		scraper.httpGet("start", LOGOUT_URL);
	};

	self.kontoliste = function() {
		var result = [];
		scraper.httpGet("start", KONTOLISTE_URL);
		var text = scraper.getText();
		var regex = /<td class="konto_blz">([\s\S]+?)<\/tr>/g;
		while (1) {
			var m = regex.exec(text);
			if (!m) break;
			var s = m[1];
			//println("kontoliste", "xx", s);
			m = s.match(/<strong>\s*(\d+)\s*\/([^>]+)<\/strong>/);
			var kontoNr = m[1];
			scraper.assertTrue("kontoNr", kontoNr);
			m = s.match(/>([^<]+)<\/a>/);
			var bezeichnung = m[1];
			scraper.assertTrue("bezeichnung", bezeichnung);
			result.push({kontoNr: kontoNr, waehrung: "EUR", bezeichnung: bezeichnung});
		}
		//println("kontoliste", "done", JSON.stringify(result));
		return result;
	};

	self.umsaetze = function() {
		scraper.followLink("gebuchteUmsaetze", /<a href="([^\"]+)">Gebuchte Ums/);

		(function() {
			var von = (function() {
				var a = scraper.data.von;
				var b = scraper.reformatDate(new Date(Date.now() - 360*24*60*60*1000));
				return a < b ? b : a;
			})();
			var day = von.substring(4, 6);
			var month = von.substring(2, 4);
			var year = "20" + von.substring(0, 2);

			var selectedAcountIndex = (function() {
				var s = scraper.findSimple("selectedAcountIndex", /<select name="accInfoReg.selectList.selectedAcountIndex"[^>]*>([\s\S]+?)<\/select>/);
				var regex = /<option\b[^>]*value="([^\"])+"[^>]*>\s*(\d+)/g;
				while (1) {
					var m = regex.exec(scraper.getText());
					scraper.assertTrue("kontoInSelect", m);
					if (m[2] === scraper.data.kontoNr) return m[1];
				}
			})();

			scraper.sendForm("accInfoReg", null, {
				"accInfoReg.visible": "Y",
				"accInfoReg.selectList.selectedAcountIndex": selectedAcountIndex,
				"accInfoReg.days": -180,
				"accInfoReg.checkbox": "S",
				"accInfoReg.iniDateChanges.day": day,
				"accInfoReg.iniDateChanges.month": month,
				"accInfoReg.iniDateChanges.year": year,
				"accInfoReg.typeTransaction": "T",
				"accInfoReg.linkButton.events.0" :"Suchen",
				"accInfoReg.linkButton.events.1": null,
			});
		})();

		var isEmpty = (function(text) {
			if (!~text.indexOf("Es sind keine Transaktionen ")) return false;
			if (!~text.indexOf(" Zeitraum vorhanden")) return false;
			return true;
		})(scraper.getText());

		if (isEmpty) return {umsaetze: []};

		scraper.followLink("export",  /<a href="([^\"]+)">Export<\/a>/);
		var text = scraper.getText();
		scraper.restorePreviousState();
		return parse(text) || {umsaetze: []};

		function reformatDate(d) {
			return d.replace(/^(\d+)\.(\d+)\.(\d+)$/, "$3$2$1");
		}

		function parse(csv) {
			var umsaetze = [];
			var i = csv.indexOf("Buchungsdatum;Wertstellung;");
			scraper.assertTrue("csvHeader", i > -1);
			csv = csv.substring(i);
			i = csv.indexOf("\n");
			if (i == -1) return;
			csv = csv.substring(i+1).trim();
			//println(csv);
			if (!csv) return;
			var lines = csv.split(/\r?\n/);
			for (var j=0; j<lines.length; ++j) {
				var line = lines[j].split(";");
				scraper.assertTrue("lineLength", line.length >= 5);
				umsaetze.push(makeUmsatz(line));
			}

			return {
				umsaetze: umsaetze,
				endSaldo: makeSaldo(lines[0].split(";"))
			};

			function makeUmsatz(line) {
				return {
					gebucht: true,
					abgerechnet: true,
					valuta: reformatDate(line[0]),
					datum: reformatDate(line[1]),
					betrag: +(line[3].replace(/[^\-,0-9]+/g, "").replace(",", ".")),
					waehrung: line[3].replace(/[^A-Z]+/g, ""),
					zweck: line.slice(5).map(function (s) {return s.trim();}).filter(function (s) {return s;}).join("@")
				};
			}

			function makeSaldo(line) {
				return {
					betrag: +(line[4].replace(/[^\-,0-9]+/g, "").replace(",", ".")),
					waehrung: line[4].replace(/[^A-Z]+/g, ""),
					datum: reformatDate(line[0])
				};
			}
		}
	};
})(window);
