if (!window) {
	throw "Unrecognized environment (no window global object).";
} else if (typeof XMLHttpRequest === "function") { // Assume running in a browser.
	window.println = console.log.bind(window.console);
} else if (typeof mpxCallback === "function") { // Assume running in moneyplex.
	if (!window.console) window.console = {};
	if (!window.console.log) window.console.log = println;
	// nothing to do
} else {
	throw "Unrecognized environment (neither a browser nor moneyplex).";
}

if (!Object.values) {
	Object.values = function(obj) {
		var keys = Object.keys(obj), i = keys.length, result = new Array(i);
		while (i--) result[i] = obj[keys[i]];
		return result;
	};
}

(function(window, undefined) {
	"use strict";
	var scraper = window.scraper = {data: {}};

	var steps = [];
	var req;
	var text;
	var lastFinalUrl;
	var _headers = '';
	var responseHeaders = {};
	var _referer;
	var _sleep = 0;

	var previousText;
	var previousLastFinalUrl;

	function newXMLHttpRequest() {
		if (typeof XMLHttpRequest === "function") return new XMLHttpRequest();
		var result = {};
		result.open = function(method, url) {
			result.method = method;
			result.url = scraper.combineUrls(lastFinalUrl, url);
		};
		result.send = function(data) {
			result.data = data || "";
			result.referer = _referer || "";
			result.sleep = _sleep || 0;
			result.headers = _headers;
			_headers = "";
			window.mpxCallback("https", result);
			lastFinalUrl = result.lastFinalUrl;
			responseHeaders = {};
			if (result.responseHeaders) {
				result.responseHeaders.split(/\r?\n/).forEach(function(s) {
					s = s.trim();
					if (!s) return;
					if (s.match(/^HTTP\//)) return;
					var m = s.match(/(.+?):\s*(.*)/);
					responseHeaders[m[1].toLowerCase()] = m[2];
				});
			}
		};
		result.setRequestHeader = function(key, value) {
			_headers = _headers + (_headers ? "\n" : "") + key + ": " + value;
		};
		return result;
	}

	scraper.starting = function(step) {
		//println("scraper.starting: " + step);
		steps.push(step);
	};

	scraper.hbciMessages = [];
	scraper.error = "";

	var htmlEntitiesMap = {
		amp: "&",
		quot: "\"",
		apos: "\'",
		lt: "<",
		gt: ">",
		auml: "ä",
		ouml: "ö",
		uuml: "ü",
		Auml: "Ä",
		Ouml: "Ö",
		Uuml: "Ü",
		szlig: "ß"
	};

	scraper.isSameKontoNr =function(a, b) {
		if (a.length !== b.length) return false;
		if (a === b) return true;
		a = a.replace(/x/gi, "*");
		b = b.replace(/x/gi, "*");
		for (var i=0; i<a.length; ++i) {
			var x = a.charAt(i);
			var y = b.charAt(i);
			if (x!==y && x!=="*" && y!=="*") return false;
		}
		return true;
	};

	scraper.acceptKontoNr =function(a) {
		return scraper.isSameKontoNr(a, scraper.data.kontoNr);
	};

	function decodeHtmlEntities(s) {
		function repl(ignored, s1, dec, hex) {
			if (hex) dec = parseInt(hex, 16);
			if (dec) return String.fromCharCode(+dec);
			switch (s1.toLowerCase()) {
				case "amp": return "&";
				case "quot": return "\"";
				case "apos": return "'";
				case "lt": return "<";
				case "gt": return ">";
			}
		}
		return s.replace(/&(?:(quot|amp|apos|lt|gt)|#([0-9]+)|#x([0-9A-F]+));/gi, repl);
	}

	scraper.decodeHtmlEntities = decodeHtmlEntities;

	function extractAttributes(text) {
		var result = {};
		text = text.replace(/>[\s\S]*/, "");
		var regex = /\b(\w+)=(?:\"([^\"]*)\"|\'([^\']*)\')/g;
		while (1) {
			var m = regex.exec(text);
			if (!m) break;
			result[m[1].toLowerCase()] = decodeHtmlEntities(m[2] || m[3]);
		}
		//println("extractAttributes", JSON.stringify(result));
		return result;
	}

	function findForms(name, predicate, overrides, submitName) {
		scraper.starting("findForms-" + name);
		predicate = predicate || function(a) {return a.name === name;};
		var result = [];
		var regex = /<form [\s\S]*?<\/form>/gi;
		while (1) {
			var m = regex.exec(scraper.getText());
			if (!m) return result;
			var a = extractAttributes(m[0]);
			var f = prepareForm(name, m[0], overrides, submitName);
			if (predicate(f)) result.push(f);
		}
	};

	scraper.findForms = findForms;

	// Collect all input and submit tags from `text`, while replacing and adding values using the `overrides` map.
	// Additionally retrieve the `action` and `method`.
	// Return a function executing the proper HTTP request.
	function prepareForm(name, text, overrides, submitName) {
		var map = {};
		var list = [];
		var url = "";
		var attr = extractAttributes(text);
		var action = attr.action;
		var method = attr.method ? attr.method.toUpperCase() : "GET";
		if (!overrides) overrides = {};
		var regex = /<(input|button)\b([^<>]*)>/gi;
		while (1) {
			var m = regex.exec(text);
			if (!m) break;
			var isSubmit = !!m[0].match(/type=[\'\"]submit[\'\"]/) || m[1] === "button";
			var isImage = !!m[0].match(/type=[\'\"]image[\'\"]/);
			var t = m[2];
			var key = t.match(/name=(?:"([^\"]*)"|'([^\']*)'|([^ \"\'>]+))/i);
			if (!key) continue;
			key = key[1] || key[2] || key[3] || "";
			if (isSubmit && submitName && key != submitName) continue;
			if (isImage) {
				record(key + ".x", t);
				record(key + ".y", t);
			} else {
				record(key, t);
			}
		}

		Object.keys(overrides).forEach(function(key) {
			if (map[key] !== undefined) return;
			var value = overrides[key];
			if (value == undefined) return;
			map[key] = value;
			list.push(key);
			list.push(value);
		});

		function record(key, t)  {
			var value = overrides[key];
			if (value === null) return;
			if (!value) {
				value = t.match(/value=(?:\"([^\"]*)\"|'([^']+)'|([^ \"\'>]+))/i);
				value = value ? value[1] || value[2] || value[3] || "" : "";
				value = decodeHtmlEntities(value);
			}
			map[key] = value;
			list.push(key);
			list.push(value);
			url += (url ? "&" : "") + encodeURIComponent(key) + "=" + encodeURIComponent(value);
		}

		function send() {
			if (method === "GET") {
				var sep = action.indexOf("?") === -1 ? "?" : "&";
				scraper.httpGet("sendForm-" + name, action + sep + url);
			} else {
				//println("MAP " + JSON.stringify(map));
				scraper.httpPost("sendForm-" + name, action, list);
			}
		}
		send.id = attr.id;
		send.text = text;
		send.name = attr.name;
		send.map = map;
		send.list = list;
		send.action = action;

		return send;
	};

	scraper.sendForm = function(name, predicate, overrides, submitName) {
		if (!overrides) overrides = {};
		var forms = findForms(name, predicate, overrides, submitName);
		scraper.assertTrue("findForms.length=" + forms.length, forms.length == 1);
		forms[0]();
	}

	scraper.followLink = function(trace, pattern) {
		var url = scraper.resolveLink(trace, pattern);
		scraper.httpGet("follow-" + trace, url);
	}

	scraper.resolveLink = function(trace, pattern) {
		var url = scraper.findSimple(trace, pattern);
		return decodeHtmlEntities(url);
	}

	scraper.combineUrls = function(base, relative) {
		var first = base || lastFinalUrl;
		if (!relative) return base;
		if (!first) return relative;
		if (~relative.indexOf("//")) return relative;
		if (relative[0] === '/') first = first.replace(/(\/\/[^/]+)\/.*/, "$1");
		var i = first.indexOf("//");
		var schema = i===-1 ? "" : first.substring(0, i+2);
		var s = i===-1 ? first : first.substring(i+2);
		var used = s.replace(/\/[^/]+$/, "");
		var result = schema + used + (relative[0]==="/" ? "" : "/") + relative;
		//println("combineUrls: " + base + " +++ " + relative + " === " + result);
		return result;
	}

	scraper.assertTrue = function(where, cond) {
		if (cond) return;
		var s = where ? "Asserting " + where + ": " : "";
		throw new Error(s + "Fehler im letzten Schritt der Folge: [" + steps + "]");
	};

	scraper.findSimple = function(trace, regex) {
		scraper.starting("find-" + trace);
		scraper.assertTrue("status", req.status == 200);
		var m = text.match(regex);
		scraper.assertTrue("match", m);
		return m[1];
	};

	scraper.getText = function() {
		return text;
	};

	scraper.setText = function(text2) {
		text = text2;
	};

	scraper.restorePreviousState = function() {
		scraper.assertTrue("previousText", previousText);
		scraper.assertTrue("previousLastFinalUrl", previousLastFinalUrl);
		text = previousText;
		lastFinalUrl = previousLastFinalUrl;
	};

	scraper.hbciDateToDate = function(s) {
		if (!s) return null;
		if (s.length === 6) s = "20" + s;
		if (s.length !== 8) return null;
		var y = +s.substring(0, 4);
		var m = +s.substring(4, 6);
		var d = +s.substring(6, 8);
		return new Date(y, m-1, d);
	};

	scraper.formatDate4 = function(date) {//!
		var d = "" + date.getDate();
		if (d.length===1) d = "0" + d;
		var m = "" + (date.getMonth() + 1);
		if (m.length===1) m = "0" + m;
		var y = "" + date.getFullYear();
		return d + "." + m + "." + y;
	};

	scraper.reformatDate = function(date, informat, outformat) {
		if (!date) throw "Undefined date";
		informat = informat ? informat.replace(/Y/g, "y") : "";
		if (!informat.match(/^(|ymd|dmy|mdy)$/)) throw "Unsupported informat";
		outformat = (function() {
			if (!outformat) return "";
			switch (outformat) {
				case "dmY": return "d.m.Y";
				case "dmy": return "d.m.y";
			}
			return outformat;
		})();

		var y, m, d;

		function computeInformat(separator) {
			if (informat) return informat;
			switch (separator) {
				case "-": return "ymd";
				case ".": return "dmy";
				case "/": return "mdy";
			}
			throw "Cannot determine informat for " + date;
		}

		function parse() {
			var matcher;
			if (date instanceof Date) {
				y = "" + date.getFullYear();
				m = "" + (date.getMonth() + 1);
				d = "" + date.getDate();
			} else if (matcher = date.match(/^(\d{1,4})([\-.\/])(\d{1,2})\2(\d{1,4})$/)) {
				var s0 = matcher[1], s1 = matcher[3], s2 = matcher[4];
				switch (computeInformat(matcher[2])) {
				case "ymd": y = s0; m = s1; d = s2; break;
				case "dmy": y = s2; m = s1; d = s0; break;
				case "mdy": y = s2; m = s0; d = s1; break;
				}
			} else if (matcher = date.match(/^(\d{2})(\d{2})(\d{2})$/)) { // hbci
				y = matcher[1];
				m = matcher[2];
				d = matcher[3];
			} else {
				throw "Unrecognized date: " + date;
			}
		}

		function normalize() {
			if (y.length === 4) y = y.replace(/^20/, "");
			while (y.length<2) y = "0" + y;
			while (m.length<2) m = "0" + m;
			while (d.length<2) d = "0" + d;
		}

		function format() {
			function format1(c) {
				switch (c) {
					case "Y": return "20" + y;
					case "y": return y;
					case "m": return m;
					case "d": return d;
				}
				return "" + c;
			}

			if (!outformat) return y + m + d;
			var result = "";
			for (var i=0; i<outformat.length; ++i) {
				result += format1(outformat.charAt(i));
			}
			return result;
		}

		parse();
		normalize();
		return format();
	}

	// TODO remove
	scraper.splitCsvLine = function(line, separator) {
		var result = [];
		var n;
		while (line) {
			if (line[0] === "'") {
				n = line.indexOf("'", 1);
				scraper.assertTrue("splitCsvLine-1", ~n);
				result.push(line.substring(1, n));
				if (line.length === n+1) break;
				line = line.substring(n+2);
			} else if (line[0] === '"') {
				n = line.indexOf('"', 1);
				scraper.assertTrue("splitCsvLine-2", ~n);
				result.push(line.substring(1, n));
				if (line.length === n+1) break;
				line = line.substring(n+2);
			} else {
				n = line.indexOf(separator);
				if (~n) {
					result.push(line.substring(0, n));
					line = line.substring(n+1);
				} else {
					result.push(line);
					break;
				}
			}
		}
		return result;
	};

	scraper.processCsv = function(separator, lineProcessor) {
		function splitLine(line) {
			var result = [];
			while (line) {
				if (line.charAt(0) === "'") {
					var n = line.indexOf("'", 1);
					scraper.assertTrue("processCsv-1", ~n);
					result.push(line.substring(1, n));
					if (line.length === n+1) break;
					line = line.substring(n+2);
				} else if (line.charAt(0) === "\"") {
					var n = line.indexOf("\"", 1);
					scraper.assertTrue("processCsv-2", ~n);
					result.push(line.substring(1, n));
					if (line.length === n+1) break;
					line = line.substring(n+2);
				} else {
					var n = line.indexOf(separator);
					if (~n) {
						result.push(line.substring(0, n));
						line = line.substring(n+1);
					} else {
						result.push(line);
						break;
					}
				}
			}
			result = result.map(function(s) {return s.trim();});
			return result;
		}

		var result = [];
		var lines = scraper.getText().split("\n");
		for (var i=0; i<lines.length; ++i) {
			var line = lines[i];
			var fields = splitLine(line);
			var o = lineProcessor(fields, i, lines);
			if (o !== undefined) result.push(o);
		}
		return result;
	}

	scraper.settings = {
		overlapDays: 1,
		speicherzeitraum: 360,
		sleep: 0,
	};

	function startVorgang(name) {
		steps = [name];
		scraper.error = "";
		scraper.hbciMessages = [];
	}

	scraper.login = function() {
		startVorgang("LOGIN");
		scraper.assertTrue("kennung", scraper.data.kennung);
		scraper.assertTrue("pin", scraper.data.pin);
		scraper.worker.login();
		scraper.starting("DONE");
	};

	scraper.logout = function() {
		startVorgang("LOGOUT");
		scraper.worker.logout();
		scraper.starting("DONE");
	};

	scraper.kontoliste = function() {
		startVorgang("KONTOLISTE");
		var result = scraper.worker.kontoliste();
		(result || []).forEach(function(e) {
			e.kontoNr = e.kontoNr.replace(/x/gi, "*");
			e.kontoinhaber = e.kontoinhaber || "";
		});
		scraper.starting("DONE");
		return result;
	};

	scraper.umsaetze = function() {
		function prepareData(data) {
			if (!data.bis) data.bis = scraper.reformatDate(new Date());

			var millisPerDay = 24*60*60*1000;

			var a = new Date();
			a.setTime(a.getTime() - scraper.settings.speicherzeitraum * millisPerDay);
			var b = scraper.hbciDateToDate(data.von) || new Date(a.getTime());
			b.setTime(b.getTime() - (scraper.settings.overlapDays - 1) * millisPerDay);
			data.von = scraper.reformatDate(a > b ? a : b);
		}

		function dump(u) {
			return "\n" + u.datum + " " + u.valuta + " " + u.betrag + " " + u.zweck;
		}

		function completeUmsaetze(list) {
			for (var i=0; i<list.length; ++i) {
				var x = list[i];
				x.eName = x.eName || "";
				x.eIban = x.eIban || "";
				x.eBic = x.eBic || "";
			}
		}

		function filterUmsaetze(list) {
			function redate(x) {
				return x.length===6 ? "20" + x : x;
			}

			var min = redate(scraper.data.von);
			println("filterUmsaetze", "len=" + list.length, "min=" + min);
			var org = list.slice();
			list.length = 0;
			for (var i=0; i<org.length; ++i) {
				var x = redate(org[i].datum);
				//println(i, x, min);
				if (x>=min || !x || !org[i].gebucht) {
					list.push(org[i]);
					println("filterUmsaetze: keeping " + JSON.stringify(org[i]));
				} else {
					println("filterUmsaetze: dropping " + JSON.stringify(org[i]));
				}
			}
		}

		function sortUmsaetze(list) {
			//println("BEFORE SORT" + list.map(dump));
			for (var i=0; i<list.length; ++i) list[i].index = -i;
			list.sort(function(a, b) {
				if (a.datum < b.datum) return -1;
				if (a.datum > b.datum) return +1;
				return a.index - b.index;
			});
			//println("AFTER SORT" + list.map(dump));
		}

		function addAnfangsSaldo(list) {
			var endSaldo = result.endSaldo;
			if (!endSaldo) return;
			var betrag = endSaldo.betrag;
			var datum = endSaldo.datum;
			var waehrung = endSaldo.waehrung;
			for (var i=0; i<list.length; ++i) {
				var u = list[i];
				betrag -= u.betrag;
				if (u.datum < datum) datum = u.datum;
			}
			result.anfangsSaldo = {betrag: betrag, waehrung: waehrung, datum: datum};
		}

		startVorgang("UMSAETZE");
		prepareData(scraper.data);
		var result = scraper.worker.umsaetze();
		scraper.starting("SORTING");
		completeUmsaetze(result.umsaetze);
		filterUmsaetze(result.umsaetze);
		sortUmsaetze(result.umsaetze);
		result.umsaetze.forEach(function(u) {
			u.zweck = u.zweck.replace(/\n/g, "@");
		});
		scraper.starting("ANFANGSSALDO");
		if (!result.anfangsSaldo) addAnfangsSaldo(result.umsaetze);
		if (result.anfangsSaldo) result.anfangsSaldo.betrag = Math.round(100*result.anfangsSaldo.betrag) / 100;
		scraper.starting("DONE");
		println("UMSAETZE", JSON.stringify(result));
		return result;
	}

	function setHeaders(req, headers) {
		if (!headers) return;
		for (var k in headers) {
			if (headers.hasOwnProperty(k)) req.setRequestHeader(k, headers[k]);
		}
	}

	function httpSend(req, data) {
		previousText = text;
		previousLastFinalUrl = lastFinalUrl;
		req.send(data);
		scraper.assertTrue("status=" + req.status, req.status == 200);
		text = req.responseText.replace(/[ \t\r]+\n/g, "\n");
	}

	scraper.setReferer = function(referer) {
		_referer = referer || "";
	};

	scraper.setSleep = function(sleep) {
		_sleep = sleep || 0;
	};

	scraper.hirmsAdd = function(message) {
		scraper.hbciMessages.push(message);
	};

	scraper.hirmsThrow = function(message) {
		scraper.error = message;
		throw "STOP " + message;
	};

	scraper.hirmsThrowPassword = function() {
		scraper.error = "Falsches Passwort";
		throw "Falsches Passwort";
	};

	scraper.getLastFinalUrl = function() {
		return lastFinalUrl;
	};

	scraper.httpGet = function(trace, url, headers) {
		scraper.starting(trace);
		req = newXMLHttpRequest();
		req.open("GET", url, false);
		setHeaders(req, headers);
		httpSend(req, null);
	};

	scraper.httpPost = function(trace, url, data, headers) {
		scraper.starting(trace);
		req = newXMLHttpRequest();
		req.open("POST", url, false);

		setHeaders(req, headers);
		var list = [];
		if (Array.isArray(data)) {
			for (var i=0; i<data.length; i+=2) {
				list.push(encodeURIComponent(data[i]) + "=" + encodeURIComponent(data[i+1]));
			}
			httpSend(req, list.join("&"));
		} else if (typeof data === "object") {
			for (var k in data) {
				if (data.hasOwnProperty(k)) list.push(encodeURIComponent(k) + "=" + encodeURIComponent(data[k]));
			}
			httpSend(req, list.join("&"));
		} else {
			httpSend(req, data);
		}
	};

	scraper.queryType = {
		TEXT: "TEXT", // show a text field
		INFO: "INFO", // show an information
		SELECT: "SELECT", // show a select
		TAN_STARTCODE: "TAN_STARTCODE", // ask for a manuall TAN
		TAN_PUSHTAN: "TAN_PUSHTAN", // ask for a push TAN
		TAN_NONE: "TAN_NONE", // wait for the Banking App
	};

	scraper.query = function(challenge, type) {
		scraper.starting("query");
		scraper.hirmsAdd("Sicherheitsabfrage: " + challenge.replace(/\n[\s\S]+/, ""));
		if (!type) type = scraper.queryType.TEXT;
		var result = {challenge: challenge, type: type};
		println("scraper.query: " + challenge, type);
		window.mpxCallback("query", result);
		if (!result.response && type!==scraper.queryType.TAN_NONE) scraper.hirmsThrow('Abbruch durch den Benutzer');
		return result.response;
	};

	scraper.getResponseHeader = function(name) {
		return responseHeaders[name.toLowerCase()] || "";
	};
})(window);
