(function(window) {
	"use strict";
	var scraper = window.scraper;
	var self = scraper.worker = {};

	var BASE_URL = "https://www.targobank.de/de";
	var START_URL = BASE_URL + "/online-banking/login.cgi";
	var UMS_URL = BASE_URL + "/banque";
	var KONTENUEBERSICHT_URL = BASE_URL + "/banque/situation_financiere.cgi";
	var LOGOUT_URL = BASE_URL + "/identification/deconnexion/deconnexion.cgi";

	var mouvements;
	function ensureMouvements() {
		if (mouvements) return;

		scraper.httpGet("kontenuebersicht", KONTENUEBERSICHT_URL);
		var text = scraper.getText();
		scraper.findSimple("find-mouvements", /(mouvements(?:_icard)?\.cgi)/);

		var list = (function() {
			var result = [];
			var regex = /(\"mouvements(?:_icard)?\.cgi\?.*)<\/a>/g;
			while (1) {
				var m = regex.exec(text);
				if (!m) break;
				result.push(m[1]);
			}
			return result;
		})();
		mouvements = list.map(function(e) {
			var m = e.match(/\"(mouvements[^\"]+)\">([^ ]+) (.*)/);
			var k = m[2];
			return {kontoNr: k, waehrung: "EUR", bezeichnung: m[3].trim(), link: m[1]};
		});;
	}

	self.bpdVersion = 2;
	self.updVersion = 1;

	self.login = function() {
		scraper.httpGet("start", START_URL);

		scraper.httpPost("login", START_URL, {
			_cm_user: scraper.data.kennung,
			flag: "password",
			_cm_pwd: scraper.data.pin,
		});

		if (scraper.getText().match("Daten sind falsch")) throw scraper.hirmsThrowPassword();
		scraper.findSimple("logged-in", /(Letzter Login am)/);
	};

	self.logout = function() {
		mouvements = null;
		scraper.httpGet("start", LOGOUT_URL);
		scraper.findSimple("logout-ok", /(Erfolgreich ausgeloggt)/i);
	};

	self.kontoliste = function() {
		var result = [];
		ensureMouvements();
		return mouvements;
	};

	self.umsaetze = function() {
		var kontoNr = scraper.data.kontoNr;
		var result = {};

		function match1(text, regex) {
			var m = text.match(regex);
			scraper.assertTrue("match1", m);
			return m[1];
		}

		function reformatDate(d) {
			return d.replace(/^(\d+)\.(\d+)\.(\d+)$/, "$3$2$1");
		}

		(function() {
			ensureMouvements();
			var list = mouvements.filter(function(e) {return scraper.acceptKontoNr(e.kontoNr);});
			println("findKonto", scraper.data.kontoNr, "->", JSON.stringify(list));
			scraper.assertTrue("findKonto-nonempty", list.length);
			scraper.assertTrue("findKonto-unique", list.length===1);
			var m = list[0];
			scraper.assertTrue("ensureMouvements", m);
			scraper.httpGet("mouvements", UMS_URL + "/" + m.link);
		})();

		result.endSaldo = (function() {
			var datum = scraper.findSimple("find-datum", /Letzte Transaktionen am (\d\d\.\d\d\.\d\d\d\d)\b/);
			//println("datum", datum);
			datum = reformatDate(datum);

			var stand = scraper.findSimple("find-kontostand", /Aktueller Kontostand<\/th>\s*<td\b[^<>]*>([^<>]+)/);
			stand = stand.replace(/ |&nbsp;/g, "").replace(/\./g, "").replace(",", ".");

			var m = stand.match(/^([-+]\d+\.\d+)([A-Z]{3})$/);
			scraper.assertTrue("endSaldo-match", m);
			var betrag = +m[1];
			var waehrung = m[2];
			return {betrag: betrag, waehrung: waehrung, datum: datum};
		})();

		var csvLink = (function() {
			var list = [];
			var required = scraper.data.kontoNr.split(/[x*]+/i).filter(function(s) {return s});
			var text = scraper.getText();
			var regex = /\"(routetelechargement.asp\?formatTelechargement=CSV&([^\"<>]+))\"/g;
			while (1) {
				var m = regex.exec(text);
				if (!m) break;
				var s = m[1];
				s = s.replace("montantCSV=2col", "montantCSV=1col");
				s = s.replace("dateCSV=US", "dateCSV=DE");
				var accept = true;
				for (var i=0; i<required.length; ++i) {
					accept &= (m[2].indexOf(required[i]) > -1);
				}
				println("csvLinks", "accept="+accept, s);
				list.push(s);
			}
			scraper.assertTrue("csvLinks-nonempty", list.length);
			return list[0];
		})();

		scraper.httpGet("csv", UMS_URL + "/" + csvLink);
		var csv = scraper.getText();

		//println("csv", csv);

		result.umsaetze = (function() {
			var list = [];
			if (csv.trim() === "Keine Bewegung auf dem Konto") return list;
			var lines = csv.split(/\r?\n/);
			for (var j=0; j<lines.length; ++j) {
				var s = lines[j].trim();
				if (!s) continue;
				var line = scraper.splitCsvLine(s, ",");
				scraper.assertTrue("lineLength", line.length >= 3);
				var umsatz = {
					gebucht: true,
					abgerechnet: true,
					valuta: reformatDate(line[0]),
					datum: null,
					zweck: line[1].replace(/\s+/g, " "),
					betrag: +(line[2].replace(",", ".")),
					waehrung: 'EUR',
				};
				umsatz.datum = umsatz.valuta; //!
				list.push(umsatz);
			}
			return list;
		})();
		//println("umsaetze", result.umsaetze);

		result.anfangsSaldo = (function() {
			var betrag = result.endSaldo.betrag;
			var datum = result.endSaldo.datum;
			for (var i=0; i<result.umsaetze.length; ++i) {
				var u = result.umsaetze[i];
				betrag -= u.betrag;
				if (u.valuta < datum) datum = u.valuta;
			}
			var waehrung = result.endSaldo.waehrung;
			return {betrag: betrag, waehrung: waehrung, datum: datum};
		})();

		return result;
	};
})(window);
