(function(window) {
	"use strict";
	var scraper = window.scraper;
	var self = scraper.worker = {};

	var BASE_URL = "https://banking.vwfs.de";
	var authorization;

	self.bpdVersion = 2;
	self.updVersion = 1;

	self.login = function() {
		scraper.httpGet("start", BASE_URL + "/PBODE-FE-UI/");
		scraper.httpPost("authenticate",
						 BASE_URL + "/PboDe.AuthenticationService.V2/authenticate",
						 JSON.stringify({username: scraper.data.kennung, password: scraper.data.pin}),
						 {"Content-Type": "application/json; charset=utf-8"});
		scraper.findSimple("Success", "Success");
		authorization = scraper.getResponseHeader("Authorization");
		scraper.assertTrue("authorization", authorization);
	};

	self.logout = function() {
		scraper.httpPost("logout",
						 BASE_URL + "/PboDe.AuthenticationService.V2/logout",
						 "",
						 {Authorization: authorization, "Content-Length": 0});
	};

	self.kontoliste = function() {
		scraper.httpGet("kontoliste",
						BASE_URL + "/PboDe.AccountListService.V2/accountlistservice/accountlist",
						{Authorization: authorization});
		authorization = scraper.getResponseHeader("Authorization");
		var json = JSON.parse(scraper.getText());
		var list = json.Accounts;
		scraper.assertTrue("list", list);
		return list.map(function(x) {return {kontoNr: x.AccountNumber, waehrung: x.Currency, bezeichnung: x.ProductName}});
	};

	self.umsaetze = function() {
		function parseDate(s) {
			return scraper.reformatDate(s.replace(/T.*/, ""));
		}

		var dateFrom = scraper.reformatDate(scraper.data.von, null, "Y-m-d") + "Z";
		var dateTo = scraper.reformatDate(scraper.data.bis, null, "Y-m-d") + "Z";
		scraper.httpGet("umsaetze",
						BASE_URL + "/PboDe.CashFlowService.V2/cashflow/list?accountNumber=" + scraper.data.kontoNr + "&dateFrom=" + dateFrom + "&dateTo=" + dateTo,
						{Authorization: authorization});
		authorization = scraper.getResponseHeader("Authorization");
		var json = JSON.parse(scraper.getText());

		var endSaldo = {datum: parseDate(json.DateToday), betrag: +json.EndBalance, waehrung: json.BalanceCurrency};

		var list = json.CashflowList;
		scraper.assertTrue("list", list);

		var umsaetze = list.map(function(x) {
			var ReferenceAccount = x.ReferenceAccount || {};
			return {
				gebucht: true,
				abgerechnet: true,
				valuta: parseDate(x.PostingDate),
				datum: parseDate(x.ValueDate),
				zweck: x.PaymentNotes.map(function(n) {return n.Text}).join("\n"),
				betrag: x.CreditAmount - x.DebitAmount,
				waehrung: x.Currency,
				eName: ReferenceAccount.AccountHolder,
				eIban: ReferenceAccount.Iban,
				eBic: ReferenceAccount.Bic,
			};
		});

		return {endSaldo: endSaldo, umsaetze: umsaetze};
	};
})(window);
